'''
Keep changing status for the indicator in HTML pages
'''
import random 
import paho.mqtt.client as mqtt
import asyncio
import websockets

status = pre_status = ""

# List of colors for indicators
colors = ["Gray", "Black", "Red", "Maroon", "Yellow", "Olive", "Lime", "Green", "Aqua", "Teal", "Blue", "Navy", "Fuchsia", "Purple"]

# Callback functions for MQTT
def on_connect(client, userdata, flags, rc):
    print("MQTT on_connect: " + mqtt.connack_string(rc))

def on_disconnect(client, userdata, rc):
    if rc != 0:
        print("MQTT Unexpected disconnection: {}".format(rc) )
    
def on_message(client, userdata, msg):
    print("MQTT onMessageArrived: " + msg.topic + " " + str(msg.payload))

def on_subscribe(client, userdata, mid, granted_qos):
	print("MQTT Subscribed: mid=" + str(mid) + " QoS=" + str(granted_qos))
    
def on_publish(client, userdata, mid):
	print("MQTT Published: mid=" + str(mid) )   

# WebSocket server asynchronous routines
async def send_status(websocket):
    global status, pre_status
    while True:
        if status != pre_status : 
            await websocket.send(status)
            print(f"WebSocket Pushed : {status}")
        await asyncio.sleep(8)   

async def websocket_routine() :
    async with websockets.serve(send_status, "localhost", 8080) :
        await asyncio.Future()  # run forever

# Refresh color asynchronous routine
async def refresh_color() :
    global status, pre_status
    while True :
        status = random.choice(colors)
        with open('status.txt', 'w') as f: f.write(status)
        print(f"Indicator color : {status}")

        if status != pre_status : 
            client.publish("ecs-status", status)  # MQTT 
            print(f"MQTT Publish : {status}")
            
        await asyncio.sleep(8)
        pre_status = status     
        
''' Start update '''

# MQTT connect (The IoT solution) 
client = mqtt.Client("pub-client-id")
client.on_connect = on_connect
client.on_message = on_message
client.on_publish = on_publish
client.on_disconnect = on_disconnect

client.connect("test.mosquitto.org", 1883, 60)
client.loop_start()

# Concurrent tasks for WebSocket server and refreshing color        
async def main():
    task1 = asyncio.create_task(websocket_routine())
    task2 = asyncio.create_task(refresh_color())
    await task1
    await task2

asyncio.run(main())
